#define SENSORMODE 1

#define nop() __asm volatile ("nop")
#if 1
nop();
#endif

#include <avr/io.h>
#include <avr/wdt.h> // здесь организована работа с ватчдогом
#include <inttypes.h>
#include <avr/interrupt.h>
#include <avr/sleep.h> // здесь описаны режимы сна
#if SENSORMODE
  #include <SHT85.h>
#endif
#include <util/delay.h>
#include <Mirf85.h>
#include <nRF24L0185.h>
//#include <MirfHardwareSpiDriver85.h>
#include <SPI85.h>

// ATtiny25/45/85 Pin map
//                                 +-\/-+
//                Reset/Ain0 (D 5) PB5  1|o   |8  Vcc
//  nRF24L01 CE, Pin3 - Ain3 (D 3) PB3  2|    |7  PB2 (D 2) Ain1 - nRF24L01 SCK, pin5
// nRF24L01 CSN, Pin4 - Ain2 (D 4) PB4  3|    |6  PB1 (D 1) pwm1 - nRF24L01 MOSI, pin7
//                                 GND  4|    |5  PB0 (D 0) pwm0 - nRF24L01 MISO, pin6
// Pinout
// CE   - Pin 2 / D3 / nRF3
// CSN  - Pin 3 / D4 / nRF4
// GND  - Pin 4
// MISO - Pin 5 / D0 / nRF7
// MOSI - Pin 6 / D1 / nRF6
// SCK  - Pin 7 / D2 / nRF5
// Vcc  - Pin 8 ( 3.3V )

volatile int wdcounter=0;
volatile int nexttick=1;
int bufferSize = 0;
struct buf {
  int temp;
  int humidity;
  int crc;
} buffer;
uint16_t counter = 0; 
uint8_t nodeID = 0;

#define CE  PB3 //ATTiny
#define CSN PB4 //ATTiny
#if SENSORMODE
  SHT1x sht1x(PB3, PB4);
#endif

void setup()
{

  pinMode(4, OUTPUT);

  Mirf.cePin = CE;
  Mirf.csnPin = CSN;
//  Mirf.spi = &MirfHardwareSpi;
  Mirf.init();

  byte RADDR[] = {
    0xe2, 0xf0, 0xf0, 0xf0, 0xf0  };
  byte TADDR[] = {
    0xe3, 0xf0, 0xf0, 0xf0, 0xf0  };

  // Get nodeID from TXADDR and mask the first byte (0xff)
  nodeID = *TADDR & 0xff; 

  Mirf.channel = 0x4c; // Same as rpi-hub and sendto_hub
  Mirf.rfsetup = 0x06; // 1Mbps, Max power
  Mirf.setRADDR(RADDR);  
  Mirf.setTADDR(TADDR);
  Mirf.config();

  // Enable dynamic payload 
  Mirf.configRegister( FEATURE, 1<<EN_DPL ); 
  Mirf.configRegister( DYNPD, 1<<DPL_P0 | 1<<DPL_P1 | 1<<DPL_P2 | 1<<DPL_P3 | 1<<DPL_P4 | 1<<DPL_P5 ); 


//  delay(100);

  // Print out register readinds for important settings
  //uint8_t rf_ch, rf_setup = 0;
  //byte tx_addr[5];
  //byte rx_addr[5];

  //Mirf.readRegister(RF_CH, &rf_ch,sizeof(rf_ch));
  //Mirf.readRegister(RF_SETUP, &rf_setup, sizeof(rf_setup));
  //Mirf.readRegister(TX_ADDR, tx_addr, sizeof(tx_addr));
  //Mirf.readRegister(RX_ADDR_P1, rx_addr, sizeof(rx_addr));

  //delay(2000);      // For serial debug to read the init config output

    wdt_reset(); // сбрасываем
    wdt_enable(9/*WDTO_1S*/); // разрешаем ватчдог 1 сек
    WDTCR |= _BV(WDIE); // разрешаем прерывания по ватчдогу. Иначе будет резет.
    sei(); // разрешаем прерывания      
    set_sleep_mode(SLEEP_MODE_PWR_DOWN); // если спать - то на полную

}

void loop()
{
  if (nexttick){
    //Mirf.powerUpTx();
    //delay(2000);
    nexttick=0;
    // Read from sensor at pin 4 (CSN)
    #if SENSORMODE
      buffer.temp = sht1x.readTemperatureC()*10;
      buffer.humidity = sht1x.readHumidity();
      buffer.crc=buffer.temp+buffer.humidity;
//      delay(10);
      pinMode(4, OUTPUT);
    #endif
    if ((buffer.temp>-100)&&(buffer.humidity>-100))
    {
      //sprintf(buffer,"T%03.0f,%02.0fZ",temp_c,humidity);
      Mirf.payload = sizeof(buffer);

      Mirf.send((byte *) &buffer);
      while( Mirf.isSending() )
      {
        delay(1);
      }
    }
    Mirf.powerDown();
  }
  sleep_enable(); // разрешаем сон
  sleep_cpu(); // спать!

} // End loop()

ISR (WDT_vect) {
    wdcounter++;
    if (wdcounter>=7){
      wdcounter=0;
      nexttick=1;
    }
    WDTCR |= _BV(WDIE); // разрешаем прерывания по ватчдогу. Иначе будет резет.
}

